@@ -38,6 +38,7 @@
 package org.qtproject.qt.android.bindings;
 
 import android.app.AlertDialog;
+import android.app.Dialog;
 import android.content.Context;
 import android.content.ContextWrapper;
 import android.content.DialogInterface;
@@ -180,7 +181,8 @@ public abstract class QtLoader {
                 // fatal error, show the error and quit
                 AlertDialog errorDialog = new AlertDialog.Builder(m_context).create();
                 errorDialog.setMessage(loaderParams.getString(ERROR_MESSAGE_KEY));
-                errorDialog.setButton(resources.getString(android.R.string.ok),
+                errorDialog.setButton(Dialog.BUTTON_POSITIVE,
+                        resources.getString(android.R.string.ok),
                         new DialogInterface.OnClickListener() {
                             @Override
                             public void onClick(DialogInterface dialog, int which) {
@@ -231,7 +233,8 @@ public abstract class QtLoader {
             int id = resources.getIdentifier("fatal_error_msg", "string",
                     packageName);
             errorDialog.setMessage(resources.getString(id));
-            errorDialog.setButton(resources.getString(android.R.string.ok),
+            errorDialog.setButton(Dialog.BUTTON_POSITIVE,
+                    resources.getString(android.R.string.ok),
                     new DialogInterface.OnClickListener() {
                         @Override
                         public void onClick(DialogInterface dialog, int which) {
@@ -361,7 +364,7 @@ public abstract class QtLoader {
                     }
                 }
 
-                if (!(new File(stylePath)).exists() && !extractOption.equals("none")) {
+                if (!extractOption.equals("none")) {
                     loaderParams.putString(EXTRACT_STYLE_KEY, stylePath);
                     loaderParams.putBoolean(EXTRACT_STYLE_MINIMAL_KEY, extractOption.equals("minimal"));
                 }
