@@ -53,8 +53,9 @@
 #include <qcryptographichash.h>
 #include <qdebug.h>
 
+#include <memory>
+
 #define CACHE_POSTFIX QLatin1String(".d")
-#define PREPARED_SLASH QLatin1String("prepared/")
 #define CACHE_VERSION 8
 #define DATA_DIR QLatin1String("data")
 
@@ -132,7 +133,7 @@ QString QNetworkDiskCache::cacheDirectory() const
     Prepared cache items will be stored in the new cache directory when
     they are inserted.
 
-    \sa QDesktopServices::CacheLocation
+    \sa QStandardPaths::CacheLocation
 */
 void QNetworkDiskCache::setCacheDirectory(const QString &cacheDir)
 {
@@ -196,7 +197,7 @@ QIODevice *QNetworkDiskCache::prepare(const QNetworkCacheMetaData &metaData)
             break;
         }
     }
-    QScopedPointer<QCacheItem> cacheItem(new QCacheItem);
+    std::unique_ptr<QCacheItem> cacheItem = std::make_unique<QCacheItem>();
     cacheItem->metaData = metaData;
 
     QIODevice *device = nullptr;
@@ -204,13 +205,13 @@ QIODevice *QNetworkDiskCache::prepare(const QNetworkCacheMetaData &metaData)
         cacheItem->data.open(QBuffer::ReadWrite);
         device = &(cacheItem->data);
     } else {
-        QString templateName = d->tmpCacheFileName();
+        QString fileName = d->cacheFileName(cacheItem->metaData.url());
         QT_TRY {
-            cacheItem->file = new QTemporaryFile(templateName, &cacheItem->data);
+            cacheItem->file = new QSaveFile(fileName, &cacheItem->data);
         } QT_CATCH(...) {
             cacheItem->file = nullptr;
         }
-        if (!cacheItem->file || !cacheItem->file->open()) {
+        if (!cacheItem->file || !cacheItem->file->open(QFileDevice::WriteOnly)) {
             qWarning("QNetworkDiskCache::prepare() unable to open temporary file");
             cacheItem.reset();
             return nullptr;
@@ -218,7 +219,7 @@ QIODevice *QNetworkDiskCache::prepare(const QNetworkCacheMetaData &metaData)
         cacheItem->writeHeader(cacheItem->file);
         device = cacheItem->file;
     }
-    d->inserting[device] = cacheItem.take();
+    d->inserting[device] = cacheItem.release();
     return device;
 }
 
@@ -250,7 +251,6 @@ void QNetworkDiskCache::insert(QIODevice *device)
 void QNetworkDiskCachePrivate::prepareLayout()
 {
     QDir helper;
-    helper.mkpath(cacheDirectory + PREPARED_SLASH);
 
     //Create directory and subdirectories 0-F
     helper.mkpath(dataDirectory);
@@ -271,19 +271,16 @@ void QNetworkDiskCachePrivate::storeItem(QCacheItem *cacheItem)
     Q_ASSERT(!fileName.isEmpty());
 
     if (QFile::exists(fileName)) {
-        if (!QFile::remove(fileName)) {
+        if (!removeFile(fileName)) {
             qWarning() << "QNetworkDiskCache: couldn't remove the cache file " << fileName;
             return;
         }
     }
 
-    if (currentCacheSize > 0)
-        currentCacheSize += 1024 + cacheItem->size();
     currentCacheSize = q->expire();
     if (!cacheItem->file) {
-        QString templateName = tmpCacheFileName();
-        cacheItem->file = new QTemporaryFile(templateName, &cacheItem->data);
-        if (cacheItem->file->open()) {
+        cacheItem->file = new QSaveFile(fileName, &cacheItem->data);
+        if (cacheItem->file->open(QFileDevice::WriteOnly)) {
             cacheItem->writeHeader(cacheItem->file);
             cacheItem->writeCompressedData(cacheItem->file);
         }
@@ -291,13 +288,15 @@ void QNetworkDiskCachePrivate::storeItem(QCacheItem *cacheItem)
 
     if (cacheItem->file
         && cacheItem->file->isOpen()
-        && cacheItem->file->error() == QFile::NoError) {
-        cacheItem->file->setAutoRemove(false);
-        // ### use atomic rename rather then remove & rename
-        if (cacheItem->file->rename(fileName))
-            currentCacheSize += cacheItem->file->size();
-        else
-            cacheItem->file->setAutoRemove(true);
+        && cacheItem->file->error() == QFileDevice::NoError) {
+        // We have to call size() here instead of inside the if-body because
+        // commit() invalidates the file-engine, and size() will create a new
+        // one, pointing at an empty filename.
+        qint64 size = cacheItem->file->size();
+        if (cacheItem->file->commit())
+            currentCacheSize += size;
+        // Delete and unset the QSaveFile, it's invalid now.
+        delete std::exchange(cacheItem->file, nullptr);
     }
     if (cacheItem->metaData.url() == lastItem.metaData.url())
         lastItem.reset();
@@ -395,7 +394,7 @@ QIODevice *QNetworkDiskCache::data(const QUrl &url)
     qDebug() << "QNetworkDiskCache::data()" << url;
 #endif
     Q_D(QNetworkDiskCache);
-    QScopedPointer<QBuffer> buffer;
+    std::unique_ptr<QBuffer> buffer;
     if (!url.isValid())
         return nullptr;
     if (d->lastItem.metaData.url() == url && d->lastItem.data.isOpen()) {
@@ -421,7 +420,7 @@ QIODevice *QNetworkDiskCache::data(const QUrl &url)
         }
     }
     buffer->open(QBuffer::ReadOnly);
-    return buffer.take();
+    return buffer.release();
 }
 
 /*!
@@ -533,7 +532,7 @@ qint64 QNetworkDiskCache::expire()
         }
     }
 
-    int removedFiles = 0;
+    [[maybe_unused]] int removedFiles = 0; // used under QNETWORKDISKCACHE_DEBUG
     qint64 goal = (maximumCacheSize() * 9) / 10;
     QMultiMap<QDateTime, QString>::const_iterator i = cacheItems.constBegin();
     while (i != cacheItems.constEnd()) {
@@ -542,16 +541,6 @@ qint64 QNetworkDiskCache::expire()
         QString name = i.value();
         QFile file(name);
 
-        if (name.contains(PREPARED_SLASH)) {
-            for (QCacheItem *item : qAsConst(d->inserting)) {
-                if (item && item->file && item->file->fileName() == name) {
-                    delete item->file;
-                    item->file = nullptr;
-                    break;
-                }
-            }
-        }
-
         qint64 size = file.size();
         file.remove();
         totalSize -= size;
@@ -604,12 +593,6 @@ QString QNetworkDiskCachePrivate::uniqueFileName(const QUrl &url)
     return pathFragment;
 }
 
-QString QNetworkDiskCachePrivate::tmpCacheFileName() const
-{
-    //The subdirectory is presumed to be already read for use.
-    return cacheDirectory + PREPARED_SLASH + QLatin1String("XXXXXX") + CACHE_POSTFIX;
-}
-
 /*!
     Generates fully qualified path of cached resource from a URL.
  */
@@ -660,7 +643,7 @@ enum
     CurrentCacheVersion = CACHE_VERSION
 };
 
-void QCacheItem::writeHeader(QFile *device) const
+void QCacheItem::writeHeader(QFileDevice *device) const
 {
     QDataStream out(device);
 
@@ -672,7 +655,7 @@ void QCacheItem::writeHeader(QFile *device) const
     out << compressed;
 }
 
-void QCacheItem::writeCompressedData(QFile *device) const
+void QCacheItem::writeCompressedData(QFileDevice *device) const
 {
     QDataStream out(device);
 
@@ -683,7 +666,7 @@ void QCacheItem::writeCompressedData(QFile *device) const
     Returns \c false if the file is a cache file,
     but is an older version and should be removed otherwise true.
  */
-bool QCacheItem::read(QFile *device, bool readData)
+bool QCacheItem::read(QFileDevice *device, bool readData)
 {
     reset();
 
@@ -722,7 +705,7 @@ bool QCacheItem::read(QFile *device, bool readData)
     if (!device->fileName().endsWith(expectedFilename))
         return false;
 
-    return metaData.isValid();
+    return metaData.isValid() && !metaData.rawHeaders().isEmpty();
 }
 
 QT_END_NAMESPACE
